use argh::FromArgs;

static WORDS: &str = include_str!("../words.txt");

#[derive(FromArgs, PartialEq, Debug)]
/// LetterQuest solver
struct App {
    #[argh(positional)]
    query: String,
}

impl App {
    /// check if word consists only of letters in query
    fn check_word(&self, word: &str) -> Option<String> {
        let mut query = self.query.clone();
        word.chars()
            .try_for_each(|c| {
                let position = query.chars().position(|i| i == c)?;
                query.remove(position);
                Some(())
            })
            .and(Some(word.to_string()))
    }
}

fn main() {
    let app: App = argh::from_env();

    let mut results = WORDS
        .lines()
        .filter_map(|line| {
            app.check_word(line)
        })
        .collect::<Vec<String>>();

    results.sort_by(|a, b| {
        a.len()
            .partial_cmp(&b.len())
            .expect("string length must be stortable")
    });
    for result in results {
        println!("matches: {} {}", result, result.len())
    }
}
